/******************************************************************************
 *
 * Copyright:
 *    (C) 2005 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates the power down mode.
 *
 *    Observe that at least the interrupt rountines must be compiled in
 *    ARM code, as opposed to THUMB code. All code is compiled in ARM code
 *    for simplicity.
 *
 *    Also observe that the startup code framwork must handle the VIC and
 *    dispatch the correct interrupt ISRs. This can be done with Embedded
 *    Artists' startup framework by setting the define: IRQ_HANDLER to 1
 *    (meaning: Load vector directly from VIC, i.e., LDR PC,[PC,#-0xFF0]).
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include "startup/config.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define CRYSTAL_FREQUENCY FOSC
#define PLL_FACTOR        PLL_MUL
#define VPBDIV_FACTOR     PBSD

#if (PLL_DIV == 1)
#define PLL_DIV_VALUE 0x00
#elif (PLL_DIV == 2)
#define PLL_DIV_VALUE 0x01
#elif (PLL_DIV == 4)
#define PLL_DIV_VALUE 0x10
#elif (PLL_DIV == 8)
#define PLL_DIV_VALUE 0x11
#endif


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);
static void initEint1(void);
static void irqEint1(void) __attribute__ ((interrupt));


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Actual ISR that is called whenever EINT1 generated an interrupt.
 *
 ****************************************************************************/
static void
irqEint1(void)
{
	//wait while EINT1 (P0.14) is active (low)
	//reset interrupt flag after every check
	while((EXTINT & 0x02) != 0)
		EXTINT = 0x02;

  //reinitialise PLL
  PLLCFG = (((PLL_MUL - 1) << 0) | (PLL_DIV_VALUE << 5));

  // enable PLL
  PLLCON  = 0x01;
  PLLFEED = 0xaa;
  PLLFEED = 0x55;

  // wait for PLL lock
  while ((PLLSTAT & (1 << 10)) == 0)
    ;

  // enable & connect PLL
  PLLCON  = 0x03;
  PLLFEED = 0xaa;
  PLLFEED = 0x55;

  //inform VIC that ISR is done
  VICVectAddr = 0x00;
}   


/*****************************************************************************
 *
 * Description:
 *    Setup PINSEL and VIC for EINT1 interrupt.
 *
 ****************************************************************************/
static void
initEint1(void)
{
	//set P0.14 to be EINT1 input (level-trigged, active low)
  PINSEL0 &= ~0x30000000;
  PINSEL0 |=  0x20000000; 

  VICIntSelect &= ~0x00008000;    //EINT1 interrupt is assigned to IRQ (not FIQ)
  VICVectCntl15 =  0x0000002F;    //enable vector interrupt for EINT1
  VICVectAddr15 = (tU32)irqEint1; //register ISR address
  EXTWAKE       =  0x00000002;    //EINT1 wakes up the processor
  EXTINT        =  0x0000000F;    //clear all interrupt flags
  VICIntEnable  =  0x00008000;    //enable EINT1 interrupt
}


/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((CRYSTAL_FREQUENCY * PLL_FACTOR)/ (1000 * VPBDIV_FACTOR));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}

  
/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2005 Embedded Artists AB");
  printf("\n*");
  printf("\n* Welcome to this program that demonstrates the power down");
  printf("\n* mode - the most power conserving mode in the LPC2xxx MCU.");
  printf("\n*");
  printf("\n*********************************************************");

  //initialize EINT1 interrupt
  initEint1();
  
  while(1)
  {
  	tU8 i;
  	
    printf("\n\nEntering power down mode in 5 seconds");

    for(i=0; i<20; i++)
    {
    	//wait 250 ms
      delayMs(250);
      printf(".");
    }
    printf("\nEINT1 (P0.14) wakes me up - now entering power down mode   ");

    PCONP = 0x00000000;   //turn off all peripherals
                          //= 0x00000800, if external memory controller is used
    PCON  = 0x00000002;   //enter power down mode    

    //power down mode entered...

    PCONP = 0x0000000c;   //turn on UART #0 and Timer #1
                          //= 0x0000080c, if external memory controller is used

    delayMs(1);           //wait 1 ms
    printf("\n\nNow I'm awake again!!!");
  }

  return 0;
}
